import os
import urllib.request
import threading
from gi.repository import Adw, Gtk, GdkPixbuf, Gio, GLib, Gdk
from bs4 import BeautifulSoup  # Не забудьте установить BeautifulSoup4

class IconLoader:
    BASE_URL = "https://download.etersoft.ru/pub/Etersoft/XimperLinux/eepm_play_gui/icons/"
    SUPPORTED_EXTENSIONS = ['png', 'jpg', 'svg']
    CACHE_DIR = os.path.join(os.path.expanduser("~"), ".cache", "ru.eepm.PlayGUI", "apps_icons")
    available_files = set()

    @classmethod
    def ensure_cache_dir(cls):
        if not os.path.exists(cls.CACHE_DIR):
            os.makedirs(cls.CACHE_DIR)

    @classmethod
    def get_cached_icon_path(cls, icon_name):
        return os.path.join(cls.CACHE_DIR, f"{icon_name}.png")  # Cache as PNG

    @classmethod
    def load_icon_async(cls, icon_name, callback):
        cls.ensure_cache_dir()  # Ensure the cache directory exists

        cached_icon_path = cls.get_cached_icon_path(icon_name)

        if os.path.exists(cached_icon_path):
            pixbuf = GdkPixbuf.Pixbuf.new_from_file(cached_icon_path)
            GLib.idle_add(callback, pixbuf)
            return

        def load_icon():
            for ext in cls.SUPPORTED_EXTENSIONS:
                icon_filename = f"{icon_name}.{ext}"
                if icon_filename not in cls.available_files:
                    continue  # Пропустить загрузку, если файл отсутствует

                icon_url = f"{cls.BASE_URL}{icon_filename}"
                try:
                    with urllib.request.urlopen(icon_url) as response:
                        input_stream = Gio.MemoryInputStream.new_from_data(response.read(), None)
                        pixbuf = GdkPixbuf.Pixbuf.new_from_stream(input_stream, None)

                        # Save the pixbuf to the local cache
                        pixbuf.savev(cached_icon_path, "png", [], [])

                        GLib.idle_add(callback, pixbuf)
                        return

                except Exception as e:
                    print(f"Failed to load {icon_name}.{ext}: {e}")

        threading.Thread(target=load_icon, daemon=True).start()

    @classmethod
    def fetch_file_list(cls):
        print("fetch file list...")
        try:
            with urllib.request.urlopen(cls.BASE_URL) as response:
                html = response.read()
                soup = BeautifulSoup(html, 'html.parser')

                # Ищем ссылки на файлы
                for link in soup.find_all('a'):
                    href = link.get('href')
                    if any(href.endswith(ext) for ext in cls.SUPPORTED_EXTENSIONS):
                        cls.available_files.add(href)

        except Exception as e:
            print(f"Failed to fetch file list: {e}")

class ApplicationRow(Adw.ActionRow):
    __gtype_name__ = 'ApplicationRow'

    def __init__(self, app, is_installed, on_toggle):
        super().__init__(title=app['name'], subtitle=app['dscr'])
        self.app = app
        self.is_installed = is_installed

        self.set_selectable(False)
        self.set_activatable(True)

        self.icon_image = Gtk.Image.new_from_icon_name("image-missing")
        self.icon_image.set_icon_size(Gtk.IconSize.LARGE)
        self.add_prefix(self.icon_image)

        IconLoader.load_icon_async(self.app['name'], self.set_icon)

        self.checkbox = Gtk.CheckButton()
        self.checkbox.add_css_class("selection-mode")
        self.checkbox.set_valign(Gtk.Align.CENTER)
        self.checkbox.set_halign(Gtk.Align.CENTER)
        self.checkbox.set_active(is_installed)
        self.checkbox.connect("toggled", self.on_checkbox_toggled)

        self.add_suffix(self.checkbox)
        self.connect("activated", lambda _: self.checkbox.activate())

        self.on_toggle = on_toggle  # Callback for when the checkbox is toggled

    def set_icon(self, pixbuf):
        self.icon_image.set_from_pixbuf(pixbuf)

    def on_checkbox_toggled(self, checkbox):
        self.on_toggle(self.app['name'], checkbox.get_active())
        self.update_row_css()  # Update the row's style when toggled

    def is_changed(self):
        """Determine if the checkbox state has changed compared to the installed state."""
        return self.checkbox.get_active() != self.is_installed

    def update_row_css(self):
        """Update the CSS classes for the row based on the checkbox state."""
        for css in ["marked-for-install", "marked-for-removal", "unchanged"]:
            self.remove_css_class(css)

        # Add the appropriate class based on the current checkbox state
        if self.is_changed():
            if self.checkbox.get_active():
                self.add_css_class("marked-for-install")
                print("marked-for-install")
            else:
                self.add_css_class("marked-for-removal")
                print("marked-for-removal")
        else:
            # self.add_css_class("unchanged")
            print("marked-unchanged")

# Загрузить список доступных файлов при запуске
IconLoader.fetch_file_list()
